﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using System.Reflection;

namespace CrystalTyper
{
    public partial class Form1 : Form
    {
        const int WM_KEYDOWN = 0x100;
        const int WM_KEYUP = 0x101;
        const int WM_SYSKEYDOWN = 0x104;
        const int WM_SYSKEYUP = 0x105;

        bool _Toggled = false;
        bool Toggled
        {
            get
            {
                return _Toggled;
            }
            set
            {
                _Toggled = value;
                TrayIcon.Icon = _Toggled ? GreenIcon : RedIcon;
                this.Icon = TrayIcon.Icon;
                ToggledLabel.Text = "to toggle between windows (" + (_Toggled ? "on" : "off") + ")";
            }
        }

        ControlInfo FocusedInfo { get; set; }

        KeyHook hook { get; set; }

        Icon RedIcon, GreenIcon;

        Keys WindowHotkey = Keys.F7;
        Keys ToggleHotkey = Keys.F8;
        bool WaitingForWindowHotkey = false;
        bool WaitingForToggleHotkey = false;

        public Form1()
        {
            InitializeComponent();
            hook = new KeyHook(KeyHookCB);

            Assembly a = Assembly.GetExecutingAssembly();
            RedIcon = new Icon(a.GetManifestResourceStream("CrystalTyper.Resources.red.ico"));
            GreenIcon = new Icon(a.GetManifestResourceStream("CrystalTyper.Resources.green.ico"));
            Toggled = false;

            TrayIcon.Visible = false;
        }

        struct ControlInfo
        {
            public IntPtr Window, FocusedControl;
        }

        private ControlInfo FocusedControlInActiveWindow()
        {
            ControlInfo ret;
            ret.Window = GetForegroundWindow();

            IntPtr ActiveWindowThread = GetWindowThreadProcessId(ret.Window, IntPtr.Zero);
            IntPtr ThisWindowThread = GetWindowThreadProcessId(this.Handle, IntPtr.Zero);

            AttachThreadInput(ActiveWindowThread, ThisWindowThread, true);
            ret.FocusedControl = GetFocus();
            AttachThreadInput(ActiveWindowThread, ThisWindowThread, false);

            return ret;
        }


        public bool KeyHookCB(uint Action, KeyHook.keyboardHookStruct key)
        {
            if (Action != WM_KEYDOWN)
                return false;

            Keys net_key = (Keys)key.vkCode;

            if (WaitingForWindowHotkey)
            {
                WindowHotkey = net_key;
                WindowButton.Text = net_key.ToString();
                WaitingForWindowHotkey = false;
                return true;
            }

            if (WaitingForToggleHotkey)
            {
                ToggleHotkey = net_key;
                ToggleHotkeyButton.Text = net_key.ToString();
                WaitingForToggleHotkey = false;
                return true;
            }


            if (WindowHotkey == net_key || ToggleHotkey == net_key)
            {
                if (WindowHotkey == net_key)
                {
                    FocusedInfo = FocusedControlInActiveWindow();
                    StringBuilder sb = new StringBuilder(256);
                    GetWindowText(FocusedInfo.Window, sb, sb.Capacity);
                    TrayIcon.Text = sb.ToString();
                    WindowName.Text = "to connect to the active window (" + sb.ToString() + ")";
                }

                if (ToggleHotkey == net_key)
                {
                    Toggled = !Toggled;
                }
                return true;
            }

            if (Toggled)
            {
                PostMessage(FocusedInfo.FocusedControl, Action, key.vkCode, 0);
                return true;
            }

            return false;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            TrayIcon.Visible = false;
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            if (FormWindowState.Minimized == this.WindowState)
            {
                this.Hide();
                TrayIcon.Visible = true;
            }
            else if (FormWindowState.Normal == this.WindowState)
            {
                TrayIcon.Visible = false;
                this.Show();
            }
        }

        private void TrayIcon_DoubleClick(object sender, EventArgs e)
        {
            TrayIcon.Visible = false;
            this.Show();
            this.WindowState = FormWindowState.Normal;
        }

        private void WindowButton_Click(object sender, EventArgs e)
        {
            WindowButton.Text = "?";
            WaitingForWindowHotkey = true;
        }

        private void ToggleHotkeyButton_Click(object sender, EventArgs e)
        {
            ToggleHotkeyButton.Text = "?";
            WaitingForToggleHotkey = true;
        }

        [DllImport("user32.dll")]
        static extern int PostMessage(IntPtr hWnd, uint Msg, uint wParam, uint lParam);

        [DllImport("user32.dll")]
        static extern IntPtr GetForegroundWindow();

        [DllImport("user32.dll")]
        static extern int GetWindowText(IntPtr hWnd, StringBuilder lpString, int nMaxCount);

        [DllImport("user32.dll")]
        static extern IntPtr GetWindowThreadProcessId(IntPtr hWnd, IntPtr ProcessId);

        [DllImport("user32.dll")]
        static extern IntPtr AttachThreadInput(IntPtr idAttach, IntPtr idAttachTo, bool fAttach);

        [DllImport("user32.dll")]
        static extern IntPtr GetFocus();
    }
}